<?php

namespace Drupal\liqpay\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

class LiqpayPaymentsForm extends FormBase{

  /**
   * @var \Drupal\liqpay\LiqPay|object|null
   */
	protected $liqPay;

	const PAYTYPES = [
		'card' => 'оплата картой',
		'liqpay' => 'через кабинет liqpay',
		'privat24' => 'через кабинет приват24',
		'masterpass' => 'через кабинет masterpass',
		'moment_part' => 'рассрочка',
		'cash' => 'наличными',
		'invoice' => 'счет на e-mail',
		'qr' => 'сканирование qr-кода'
	];
 
	const STATUSES = [
		'liqpay_new' => 'Новый платеж',
		'error' => 'Неуспешный платеж. Некорректно заполнены данные',
		'failure' => 'Неуспешный платеж',
		'reversed' => 'Платеж возвращен',
		'subscribed' => 'Подписка успешно оформлена',
		'success' => 'Успешный платеж',
		'unsubscribed' => 'Подписка успешно деактивирована',
		'3ds_verify' => 'Требуется 3DS верификация. Для завершения платежа, требуется выполнить 3ds_verify',
		'captcha_verify' => 'Ожидается подтверждение captcha',
		'cvv_verify' => 'Требуется ввод CVV карты отправителя. Для завершения платежа, требуется выполнить cvv_verify',
		'ivr_verify' => 'Ожидается подтверждение звонком ivr',
		'otp_verify' => 'Требуется OTP подтверждение клиента. OTP пароль отправлен на номер телефона Клиента. Для завершения платежа, требуется выполнить otp_verify',
		'password_verify' => 'Ожидается подтверждение пароля приложения Приват24',
		'phone_verify' => 'Ожидается ввод телефона клиентом. Для завершения платежа, требуется выполнить phone_verify',
		'pin_verify' => 'Ожидается подтверждение pin-code',
		'receiver_verify' => 'Требуется ввод данных получателя. Для завершения платежа, требуется выполнить receiver_verify',
		'sender_verify' => 'Требуется ввод данных отправителя. Для завершения платежа, требуется выполнить sender_verify',
		'senderapp_verify' => 'Ожидается подтверждение в приложении SENDER',
		'wait_qr' => 'Ожидается сканирование QR-кода клиентом',
		'wait_sender' => 'Ожидается подтверждение оплаты клиентом в приложении Privat24/SENDER',
		'cash_wait' => 'Ожидается оплата наличными в ТСО',
		'hold_wait' => 'Сумма успешно заблокирована на счету отправителя',
		'invoice_wait' => 'Инвойс создан успешно, ожидается оплата',
		'prepared' => 'Платеж создан, ожидается его завершение отправителем',
		'processing' => 'Платеж обрабатывается',
		'wait_accept' => 'Деньги с клиента списаны, но магазин еще не прошел проверку. Если магазин не пройдет активацию в течение 180 дней, платежи будут автоматически отменены',
		'wait_card' => 'Не установлен способ возмещения у получателя',
		'wait_compensation' => 'Платеж успешный, будет зачислен в ежесуточной проводке',
		'wait_lc' => 'Аккредитив. Деньги с клиента списаны, ожидается подтверждение доставки товара',
		'wait_reserve' => 'Средства по платежу зарезервированы для проведения возврата по ранее поданной заявке',
		'wait_secure' => 'Платеж на проверке',
		'sandbox' => 'Тестовый платеж',
	];

  /**
   * {@inheritdoc}
   */
	public function __construct(){
		$this->liqPay = \Drupal::getContainer()->get('LiqPay');
	}

  /**
   * @return string
   */
	public function getFormId(){
		return 'liqpay_payments';
	}

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array
   */
	public function buildForm(array $form, FormStateInterface $form_state){
		$form['#title'] = $this->liqPay->t('Payments');
		$form['filter'] = [
			'#type' => 'details',
			'#title' => $this->liqPay->t('Filter')
		];
		$form['filter']['status'] = [
			'#type' => 'select',
			'#title' => $this->liqPay->t('Status'),
			'#options' => $this->getOptionsStatus(),
			'#empty_option' => '',
			'#default_value' => @$_GET['status']
		];
		$form['filter']['submit'] = [
			'#type' => 'submit',
			'#value' => $this->liqPay->t('Search')
		];
		$form['table'] = [
			'#type' => 'table',
			'#header' => [
				[
					'data' => 'ID',
					'field' => 'l.id',
					'sort' => 'desc'
				], [
					'data' => $this->liqPay->t('Created'),
					'field' => 'l.created'
				], [
					'data' => 'NID',
					'field' => 'l.nid'
				], [
					'data' => 'SID',
					'field' => 'l.sid'
				], [
					'data' => $this->liqPay->t('User ID'),
					'field' => 'l.uid'
				], [
					'data' => $this->liqPay->t('Amount'),
					'field' => 'l.amount'
				],
				[
					'data' => $this->liqPay->t('Status'),
					'field' => 'l.status'
				], [
					'data' => $this->liqPay->t('Payment time'),
					'field' => 'l.paytime'
				],
				$this->liqPay->t('Payment type')
			]
		];
		foreach ($this->getPays($form['table']['#header']) as $row){

			$row->data = !empty($row->data) ? unserialize($row->data) : [];

			$form['table'][$row->id]['id'] = [
				'#markup' => $row->id
			];
			$form['table'][$row->id]['created'] = [
				'#markup' => date('d.m.Y H:i', $row->created)
			];
			$form['table'][$row->id]['nid'] = [
				'#markup' => !empty($row->nid) ? $row->nid : '- - -'
			];
			$form['table'][$row->id]['sid'] = [
				'#markup' => !empty($row->sid) ? $row->sid : '- - -'
			];
			$form['table'][$row->id]['user'] = [
				'#markup' => !empty($row->uid) ? $row->uid : '- - -'
			];
			$form['table'][$row->id]['amount'] = [
				'#markup' => $row->amount.' '.$row->currency
			];
			$form['table'][$row->id]['status'] = [
				'#markup' => $this->getPayStatus($row)
			];
			$form['table'][$row->id]['paytime'] = [
				'#markup' => !empty($row->paytime) ? date('d.m.Y H:i', $row->paytime) : '- - -'
			];
			$form['table'][$row->id]['pay_type'] = [
				'#markup' => $this->getPayType($row)
			];
			if($row->status == $this->liqPay::LIQPAY_INSERT_STATUS){
				$form['table'][$row->id]['paytime']['#markup'] = Url::fromRoute('liqpay.pages', [
					'page_type' => 'pay'
				], [
					'absolute' => TRUE,
					'query' => [
						'pay_id' => $row->id
					]
				])->toString();
			}
		}
		$form['pager'] = [
			'#type' => 'pager'
		];
		return $form;
	}

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return void
   */
	public function validateForm(array &$form, FormStateInterface $form_state) {
		parent::validateForm($form, $form_state);
	}

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return void
   */
	public function submitForm(array &$form, FormStateInterface $form_state){
		$query = $_GET;
		$query['status'] = $form_state->getValue('status');
		$url = Url::fromRoute('liqpay.payments', [], [
			'query' => $query
		]);
		$form_state->setRedirectUrl($url);
	}

  /**
   * @param $header
   *
   * @return mixed
   */
	public function getPays($header){
		$query = \Drupal::database()->select('payments_liqpay', 'l');
		$query->fields('l');
		if(!empty($_GET['status'])){
			$query->condition('l.status', $_GET['status']);
		}
		$table_sort = $query->extend('Drupal\Core\Database\Query\TableSortExtender')->orderByHeader($header);
		$pager = $table_sort->extend('Drupal\Core\Database\Query\PagerSelectExtender')->limit(50);
		return $pager->execute()->fetchAll();
	}

  /**
   * @param $row
   *
   * @return mixed|string
   */
	public function getPayType($row){
		$paytype = !empty($row->data['paytype']) ? $row->data['paytype'] : '- - -';
		if(!empty(self::PAYTYPES[$paytype])){
			$paytype = self::PAYTYPES[$paytype];
		}
		return $paytype;
	}

  /**
   * @param $row
   *
   * @return string
   */
	public function getPayStatus($row){
		$status = !empty($row->status) ? $row->status : '- - -';
		if(!empty(self::STATUSES[$status])){
			$status = self::STATUSES[$status];
		}
		return $status;
	}

  /**
   * @return array
   */
	public function getOptionsStatus(){
		$query = \Drupal::database()->select('payments_liqpay', 'l');
		$query->fields('l', ['status', 'status']);
		$options = $query->execute()->fetchAllKeyed();
		foreach ($options as $key => $value) {
			if(!empty(self::STATUSES[$key])){
				$options[$key] = self::STATUSES[$key];
			}
		}
		return $options;
	}
}