<?php

namespace Drupal\liqpay\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use stdClass;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\liqpay\API\LiqPayApi;
use Drupal\Core\Url;

class PaymentForm extends FormBase{

  /**
   * @var \Drupal\liqpay\LiqPay|object|null
   */
	protected $liqPay;

  /**
   * {@inheritdoc}
   */
	function __construct(){
		$this->liqPay = \Drupal::getContainer()->get('LiqPay');
	}

  /**
   * @return string
   */
	public function getFormId(){
		return 'payment_form';
	}

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param $payment
   *
   * @return array
   */
	public function buildForm(array $form, FormStateInterface $form_state, $payment = []){
		if(empty($payment)){
			throw new NotFoundHttpException();
		}
		$form['#id'] = 'payment_form';
		$form['#attached']['library'][] = 'liqpay/css';
		$form['info'] = [
			'#type' => 'inline_template',
			'#template' => '<div class="sum"><label>{{ label }}:</label> {{ sum }} {{ currency }}</div>',
			'#context' => [
				'label' => $this->liqPay->t('Sum'),
			]
		];
		$form['actions'] = [
			'#type' => 'actions',
			'submit' => [
				'#type' => 'submit',
				'#value' => $this->liqPay->t('Pay')
			]
		];

		$payment->result_url = Url::fromRoute('liqpay.pages', ['page_type' => 'payment_result'],['absolute' => TRUE])->toString();
		$payment->callback_url = Url::fromRoute('liqpay.pages', ['page_type' => 'api'],['absolute' => TRUE])->toString();

    // Payment alter
		$this->basketPaymentFormAlter($form, $form_state, $payment);

		$form['info']['#context']['currency'] = $form['#params']['currency'];
		$form['info']['#context']['sum'] = $form['#params']['amount'];
		return $form;
	}

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return void
   */
	public function submitForm(array &$form, FormStateInterface $form_state){}

  /**
   * @param $form
   * @param $form_state
   * @param $payment
   *
   * @return void
   */
	public function basketPaymentFormAlter(&$form, &$form_state, $payment){
		$langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
		$config = $this->liqPay->getConfig();
    
    $language = 'en';
    switch(\Drupal::languageManager()->getCurrentLanguage()->getId()) {
      case'uk':
      case'ru':
        $language = \Drupal::languageManager()->getCurrentLanguage()->getId();
        break;
    }

		$form['#params'] = [
			'action' => $this->liqPay::LIQPAY_ACTION,
			'amount' => $payment->amount,
			'currency' => $payment->currency,
			'description' => !empty($config['description'][$langcode]) ? $config['description'][$langcode] : 'Payment',
			'order_id' => $payment->id.'-'.time().rand(0, 999),
			'version' => $this->liqPay::LIQPAY_VERSION,
			'server_url' => $payment->callback_url,
			'result_url' => $payment->result_url,
      'language' => $language
		];
		
		if(\Drupal::getContainer()->has('Basket') && !empty($config['rro']) && !empty($payment->nid)) {

      $order = \Drupal::getContainer()->get('Basket')->orders(NULL, $payment->nid)->load();
			
			if(!empty($order->items)) {
				$form['#params']['rro_info']['goods'] = [];
				foreach ($order->items as $item) {
					$form['#params']['rro_info']['goods'][$item->id] = new stdClass();
					$form['#params']['rro_info']['goods'][$item->id]->good = new stdClass();
					$form['#params']['rro_info']['goods'][$item->id]->good->code = (string) $item->nid;
					$form['#params']['rro_info']['goods'][$item->id]->good->name = (string) ( $item->node_fields['title'] ?? $item->nid );
					$form['#params']['rro_info']['goods'][$item->id]->good->barcode = (string) $item->nid;
					
					$price = !empty($item->price) ? $item->price : 0;
					if(!empty($item->discount['percent'])){
						$price = $price-($price/100*$item->discount['percent']);
					}
					$form['#params']['rro_info']['goods'][$item->id]->good->price = round($price*100, 0);
					$form['#params']['rro_info']['goods'][$item->id]->quantity = round($item->count*1000, 0);
					$form['#params']['rro_info']['goods'][$item->id]->is_return = FALSE;
				}
				$form['#params']['rro_info']['goods'] = array_values($form['#params']['rro_info']['goods']);
				$form['#params']['rro_info']['delivery'] = new stdClass();
				$form['#params']['rro_info']['delivery']->email = NULL;
			}
		}
		
		// Alter
		\Drupal::moduleHandler()->alter('liqpay_payment_params', $form['#params'], $payment, $config);

		if(empty($form['#params']['rro_info']['delivery']->email)) {
			unset($form['#params']['rro_info']['delivery']);
		}
		$liqpay = new LiqPayApi(
			$config['public_key'],
			$config['private_key']
		);
		$liqpay->form_alter($form, $form['#params']);

		$_SESSION['liqpay_last_pay'] = $payment->id;
	}
}
