<?php

namespace Drupal\liqpay;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\liqpay\API\LiqPayApi;

class LiqPay{

	const LIQPAY_INSERT_STATUS = 'liqpay_new';
	const LIQPAY_SUCCESS_STATUS = 'success|sandbox|subscribed|unsubscribed|hold_wait';
	const LIQPAY_ACTION = 'pay';
	const LIQPAY_VERSION = '3';
 
	public function load($params){
		$query = \Drupal::database()->select('payments_liqpay', 'l');
		$query->fields('l');
		if(!empty($params['id'])) $query->condition('l.id', $params['id']);
		if(!empty($params['nid'])) $query->condition('l.nid', $params['nid']);
		if(!empty($params['sid'])) $query->condition('l.sid', $params['sid']);
		if(empty($params['data'])) $params['data'] = [];
		$payment = $query->execute()->fetchObject();

		if(!empty($params['create_new']))$payment = FALSE;
		if(empty($payment) && !empty($params['amount'])){
			$payment = (object)[
				'nid' => !empty($params['nid']) ? $params['nid'] : NULL,
				'sid'	=> !empty($params['sid']) ? $params['sid'] : NULL,
				'uid'	=> \Drupal::currentUser()->id(),
				'created'	=> time(),
				'paytime'	=> NULL,
				'amount' => $params['amount'],
				'currency' => !empty($params['currency']) ? $params['currency'] : \Drupal::config('liqpay.settings')->get('config.currency'),
				'status' => $this::LIQPAY_INSERT_STATUS,
				'data' => serialize($params['data'])
			];
			$payment->id = \Drupal::database()->insert('payments_liqpay')
				->fields((array)$payment)
				->execute();
		}
		return $payment;
	}
	
	public function update($payment){
		if(!empty($payment->id)){
			$updateFields = (array)$payment;
			unset($updateFields['id']);
			\Drupal::database()->update('payments_liqpay')
				->fields($updateFields)
				->condition('id', $payment->id)
				->execute();
		}
	}
	
	public function getConfig(){
		$config = \Drupal::config('liqpay.settings')->get('config');
		if(!empty($config['sandbox'])){
			$config['public_key'] = $config['public_key_sandbox'];
			$config['private_key'] = $config['private_key_sandbox'];
		}
		return $config;
	}
	
	public function getStatus($orderId){
		$config = $this->getConfig();
		$liqpay = new LiqPayApi(
			$config['public_key'],
			$config['private_key']
		);
		return $liqpay->api('request', [
			'action' => 'status',
			'version' => $this::LIQPAY_VERSION,
			'order_id' => $orderId
		]);
	}
	
	public function t($string, $args = []){
		return new TranslatableMarkup($string, $args, ['context' => 'liqpay']);
	}
  
  /**
   * @return [const_name => 'value', ...]
   */
  static function getConstants(){
    $refl = new \ReflectionClass(__CLASS__);
    return $refl->getConstants();
  }

  /**
   * @param $data
   *
   * @return string|null
   */
  public function getPayIdByData($data) {
    $orderId = NULL;

    //Alter
    \Drupal::getContainer()->get('module_handler')->alter('liqpay_get_order_id_by_data', $orderId, $data);

    if(is_null($orderId)) {
      @[$orderId, $rand] = explode('-', $data['order_id']);
    }
    return $orderId;
  }
}
